@@ -180,6 +180,24 @@ class SampleExtensionGroup < Asciidoctor::Extensions::Group
   end
 end
 
+def create_cat_in_sink_block_macro
+  Asciidoctor::Extensions.create do
+    block_macro do
+      named :cat_in_sink
+      process do |parent, target, attrs|
+        image_attrs = {}
+        unless target.nil_or_empty?
+          image_attrs['target'] = %(cat-in-sink-day-#{target}.png)
+        end
+        if (alt = attrs.delete 1)
+          image_attrs['alt'] = alt
+        end
+        create_image_block parent, image_attrs
+      end
+    end
+  end
+end
+
 context 'Extensions' do
   context 'Register' do
     test 'should register extension group class' do
@@ -1161,7 +1179,6 @@ sample content
       end
     end
 
-
     test 'should append docinfo to document' do
       begin
         Asciidoctor::Extensions.register do
@@ -1181,5 +1198,42 @@ sample content
         Asciidoctor::Extensions.unregister_all
       end
     end
+
+    test 'should raise an exception if mandatory target attribute is not provided for image block' do
+      input = <<-EOS
+.Cat in Sink?
+cat_in_sink::[]
+      EOS
+      exception = assert_raises ArgumentError do
+        render_embedded_string input, :extension_registry => create_cat_in_sink_block_macro
+      end
+      assert_match(/target attribute is required/, exception.message)
+    end
+
+    test 'should assign alt attribute to image block if alt is not provided' do
+      input = <<-EOS
+.Cat in Sink?
+cat_in_sink::25[]
+      EOS
+      doc = document_from_string input, :header_footer => false, :extension_registry => create_cat_in_sink_block_macro
+      image = doc.blocks[0]
+      assert_equal 'cat in sink day 25', (image.attr 'alt')
+      assert_equal 'cat in sink day 25', (image.attr 'default-alt')
+      output = doc.convert
+      assert_includes output, '<img src="cat-in-sink-day-25.png" alt="cat in sink day 25">'
+    end
+
+    test 'should create an image block if mandatory attributes are provided' do
+      input = <<-EOS
+.Cat in Sink?
+cat_in_sink::30[cat in sink (yes)]
+      EOS
+      doc = document_from_string input, :header_footer => false, :extension_registry => create_cat_in_sink_block_macro
+      image = doc.blocks[0]
+      assert_equal 'cat in sink (yes)', (image.attr 'alt')
+      refute(image.attr? 'default-alt')
+      output = doc.convert
+      assert_includes output, '<img src="cat-in-sink-day-30.png" alt="cat in sink (yes)">'
+    end
   end
 end
